@@ -24,7 +24,7 @@
 //!
 //!     let interface = Interface::new(config, center).unwrap();
 //!
-//!     let mut topic = interface.subscribe(&"example".to_string().to_address().unwrap());
+//!     let mut topic = interface.subscribe(&"example".to_string().to_address());
 //!
 //!     let _ = topic.broadcast("hello world".as_bytes().to_vec());
 //! }
@@ -147,7 +147,7 @@ impl Interface {
     /// processed.
     pub fn subscribe(self, addr: &Address) -> Topic {
         let (c1, c2) = Channel::new();
-        let local = Topic::new(addr.clone(), c1, Vec::new());
+        let local = Topic::new(addr.clone(), c1, Vec::new(), self.center.public.clone());
         let remote = Simple::new(addr.clone(), c2);
         let _ = self.switch.send(InterfaceAction::Subscribe(remote));
         local
